@@ -18,6 +18,8 @@
 #ifndef GLOBAL_H
 #define GLOBAL_H
 
+#include <qwindowdefs.h>
+
 #define MOUSE_MARGINS 10
 
 #define DPP_BEGIN_NAMESPACE namespace deepin_platform_plugin {
@@ -31,6 +33,7 @@
 #define DEFINE_CONST_CHAR(Name) const char Name[] = "_d_" #Name
 
 DEFINE_CONST_CHAR(useDxcb);
+DEFINE_CONST_CHAR(useDwayland);
 DEFINE_CONST_CHAR(useNativeDxcb);
 DEFINE_CONST_CHAR(redirectContent);
 DEFINE_CONST_CHAR(netWmStates);
@@ -56,20 +59,24 @@ DEFINE_CONST_CHAR(popupSystemWindowMenu);
 DEFINE_CONST_CHAR(groupLeader);
 DEFINE_CONST_CHAR(noTitlebar);
 DEFINE_CONST_CHAR(enableGLPaint);
+DEFINE_CONST_CHAR(windowInWorkSpace);
 
 // functions
 DEFINE_CONST_CHAR(setWmBlurWindowBackgroundArea);
 DEFINE_CONST_CHAR(setWmBlurWindowBackgroundPathList);
 DEFINE_CONST_CHAR(setWmBlurWindowBackgroundMaskImage);
+DEFINE_CONST_CHAR(setWmWallpaperParameter);
 DEFINE_CONST_CHAR(hasBlurWindow);
 DEFINE_CONST_CHAR(hasComposite);
 DEFINE_CONST_CHAR(hasNoTitlebar);
 DEFINE_CONST_CHAR(hasWindowAlpha);
+DEFINE_CONST_CHAR(hasWallpaperEffect);
 DEFINE_CONST_CHAR(windowManagerName);
 DEFINE_CONST_CHAR(connectWindowManagerChangedSignal);
 DEFINE_CONST_CHAR(connectHasBlurWindowChanged);
 DEFINE_CONST_CHAR(connectHasCompositeChanged);
 DEFINE_CONST_CHAR(connectHasNoTitlebarChanged);
+DEFINE_CONST_CHAR(connectHasWallpaperEffectChanged);
 DEFINE_CONST_CHAR(getWindows);
 DEFINE_CONST_CHAR(windowFromPoint);
 DEFINE_CONST_CHAR(getCurrentWorkspaceWindows);
@@ -89,10 +96,16 @@ DEFINE_CONST_CHAR(setWindowGroup);
 DEFINE_CONST_CHAR(clientLeader);
 DEFINE_CONST_CHAR(enableDxcb);
 DEFINE_CONST_CHAR(isEnableDxcb);
+DEFINE_CONST_CHAR(enableDwayland);
+DEFINE_CONST_CHAR(isEnableDwayland);
 DEFINE_CONST_CHAR(setEnableNoTitlebar);
+DEFINE_CONST_CHAR(setWindowRadius);
 DEFINE_CONST_CHAR(isEnableNoTitlebar);
 DEFINE_CONST_CHAR(buildNativeSettings);
 DEFINE_CONST_CHAR(clearNativeSettings);
+DEFINE_CONST_CHAR(setWMClassName);
+DEFINE_CONST_CHAR(splitWindowOnScreen);
+DEFINE_CONST_CHAR(supportForSplittingWindow);
 
 // others
 DEFINE_CONST_CHAR(WmWindowTypes);
@@ -106,4 +119,7 @@ enum DeviceType {
     MouseDevice
 };
 
+class QWindow;
+QWindow * fromQtWinId(WId id);
+
 #endif // GLOBAL_H
