@@ -1,37 +1,31 @@
 /****************************************************************************
 **
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
+** In addition, as a special exception, Digia gives you certain additional
+** rights. These rights are described in the Digia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,10 +35,7 @@
 #define QXCBCONNECTION_H
 
 #include <xcb/xcb.h>
-#include <xcb/randr.h>
 
-#include <QtGui/private/qtguiglobal_p.h>
-#include "qxcbexport.h"
 #include <QHash>
 #include <QList>
 #include <QMutex>
@@ -54,24 +45,20 @@
 #include <QVarLengthArray>
 #include <qpa/qwindowsysteminterface.h>
 #include <QtCore/QLoggingCategory>
-#include <QtCore/private/qglobal_p.h>
-
-#include <cstdlib>
-#include <memory>
 
 // This is needed to make Qt compile together with XKB. xkb.h is using a variable
 // which is called 'explicit', this is a reserved keyword in c++
-#if QT_CONFIG(xkb)
+#ifndef QT_NO_XKB
 #define explicit dont_use_cxx_explicit
 #include <xcb/xkb.h>
 #undef explicit
 #endif
 
-#if QT_CONFIG(tabletevent)
+#ifndef QT_NO_TABLETEVENT
 #include <QTabletEvent>
 #endif
 
-#if QT_CONFIG(xinput2)
+#if XCB_USE_XINPUT2
 #include <X11/extensions/XI2.h>
 #ifdef XIScrollClass
 #define XCB_USE_XINPUT21    // XI 2.1 adds smooth scrolling support
@@ -79,22 +66,17 @@
 #define XCB_USE_XINPUT22    // XI 2.2 adds multi-point touch support
 #endif
 #endif
-#endif // QT_CONFIG(xinput2)
-
+struct XInput2TouchDeviceData;
+#endif
 struct xcb_randr_get_output_info_reply_t;
 
+//#define Q_XCB_DEBUG
+
 QT_BEGIN_NAMESPACE
 
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInput)
 Q_DECLARE_LOGGING_CATEGORY(lcQpaXInputDevices)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaXInputEvents)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaScreen)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaEvents)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaXcb)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaPeeker)
-Q_DECLARE_LOGGING_CATEGORY(lcQpaKeyboard)
 
-class QXcbVirtualDesktop;
 class QXcbScreen;
 class QXcbWindow;
 class QXcbDrag;
@@ -103,7 +85,6 @@ class QXcbClipboard;
 class QXcbWMSupport;
 class QXcbNativeInterface;
 class QXcbSystemTrayTracker;
-class QXcbGlIntegration;
 
 namespace QXcbAtom {
     enum Atom {
@@ -122,7 +103,6 @@ namespace QXcbAtom {
         WM_STATE,
         WM_CHANGE_STATE,
         WM_CLASS,
-        WM_NAME,
 
         // Session management
         WM_CLIENT_LEADER,
@@ -207,7 +187,6 @@ namespace QXcbAtom {
         _KDE_NET_WM_WINDOW_TYPE_OVERRIDE,
 
         _KDE_NET_WM_FRAME_STRUT,
-        _NET_FRAME_EXTENTS,
 
         _NET_STARTUP_INFO,
         _NET_STARTUP_INFO_BEGIN,
@@ -274,7 +253,6 @@ namespace QXcbAtom {
         AbsMTPositionY,
         AbsMTTouchMajor,
         AbsMTTouchMinor,
-        AbsMTOrientation,
         AbsMTPressure,
         AbsMTTrackingID,
         MaxContacts,
@@ -300,8 +278,6 @@ namespace QXcbAtom {
         _COMPIZ_DECOR_PENDING,
         _COMPIZ_DECOR_REQUEST,
         _COMPIZ_DECOR_DELETE_PIXMAP,
-        _COMPIZ_TOOLKIT_ACTION,
-        _GTK_LOAD_ICONTHEMES,
 
         NPredefinedAtoms,
 
@@ -319,7 +295,7 @@ class QXcbEventReader : public QThread
 public:
     QXcbEventReader(QXcbConnection *connection);
 
-    void run() override;
+    void run();
 
     QXcbEventArray *lock();
     void unlock();
@@ -340,6 +316,9 @@ private:
     QMutex m_mutex;
     QXcbEventArray m_events;
     QXcbConnection *m_connection;
+
+    typedef xcb_generic_event_t * (*XcbPollForQueuedEventFunctionPointer)(xcb_connection_t *c);
+    XcbPollForQueuedEventFunctionPointer m_xcb_poll_for_queued_event;
 };
 
 class QXcbWindowEventListener
@@ -362,10 +341,7 @@ public:
     virtual void handleFocusInEvent(const xcb_focus_in_event_t *) {}
     virtual void handleFocusOutEvent(const xcb_focus_out_event_t *) {}
     virtual void handlePropertyNotifyEvent(const xcb_property_notify_event_t *) {}
-#if QT_CONFIG(xinput2)
-    virtual void handleXIMouseEvent(xcb_ge_event_t *, Qt::MouseEventSource = Qt::MouseEventNotSynthesized) {}
-    virtual void handleXIEnterLeave(xcb_ge_event_t *) {}
-#endif
+
     virtual QXcbWindow *toWindow() { return 0; }
 };
 
@@ -384,20 +360,17 @@ private:
 };
 
 class QAbstractEventDispatcher;
-class Q_XCB_EXPORT QXcbConnection : public QObject
+class QXcbConnection : public QObject
 {
     Q_OBJECT
 public:
-    QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, xcb_visualid_t defaultVisualId, const char *displayName = 0);
+    QXcbConnection(QXcbNativeInterface *nativeInterface, bool canGrabServer, const char *displayName = 0);
     ~QXcbConnection();
 
     QXcbConnection *connection() const { return const_cast<QXcbConnection *>(this); }
-    bool isConnected() const;
 
-    const QList<QXcbVirtualDesktop *> &virtualDesktops() const { return m_virtualDesktops; }
     const QList<QXcbScreen *> &screens() const { return m_screens; }
     int primaryScreenNumber() const { return m_primaryScreenNumber; }
-    QXcbVirtualDesktop *primaryVirtualDesktop() const { return m_virtualDesktops.value(m_primaryScreenNumber); }
     QXcbScreen *primaryScreen() const;
 
     inline xcb_atom_t atom(QXcbAtom::Atom atom) const { return m_allAtoms[atom]; }
@@ -410,15 +383,6 @@ public:
     const xcb_setup_t *setup() const { return m_setup; }
     const xcb_format_t *formatForDepth(uint8_t depth) const;
 
-    bool imageNeedsEndianSwap() const
-    {
-#if Q_BYTE_ORDER == Q_BIG_ENDIAN
-        return m_setup->image_byte_order != XCB_IMAGE_ORDER_MSB_FIRST;
-#else
-        return m_setup->image_byte_order != XCB_IMAGE_ORDER_LSB_FIRST;
-#endif
-    }
-
     QXcbKeyboard *keyboard() const { return m_keyboard; }
 
 #ifndef QT_NO_CLIPBOARD
@@ -430,82 +394,65 @@ public:
 
     QXcbWMSupport *wmSupport() const { return m_wmSupport.data(); }
     xcb_window_t rootWindow();
-    xcb_window_t clientLeader();
-
-    bool hasDefaultVisualId() const { return m_defaultVisualId != UINT_MAX; }
-    xcb_visualid_t defaultVisualId() const { return m_defaultVisualId; }
-
-#if QT_CONFIG(xcb_xlib)
-    void *xlib_display() const;
-    void *createVisualInfoForDefaultVisualId() const;
+#ifdef XCB_USE_XLIB
+    void *xlib_display() const { return m_xlib_display; }
 #endif
+
+#ifdef XCB_USE_EGL
+    bool hasEgl() const;
+#endif
+#if defined(XCB_USE_EGL)
+    void *egl_display() const { return m_egl_display; }
+#endif
+#if defined(XCB_USE_XINPUT2)
+    void xi2Select(xcb_window_t window);
+#endif
+#ifdef XCB_USE_XINPUT21
+    bool isUsingXInput21() { return m_xi2Enabled && m_xi2Minor >= 1; }
+#else
+    bool isUsingXInput21() { return false; }
+#endif
+
+
     void sync();
 
     void handleXcbError(xcb_generic_error_t *error);
-    void printXcbError(const char *message, xcb_generic_error_t *error);
     void handleXcbEvent(xcb_generic_event_t *event);
-    void printXcbEvent(const QLoggingCategory &log, const char *message,
-                       xcb_generic_event_t *event) const;
 
     void addWindowEventListener(xcb_window_t id, QXcbWindowEventListener *eventListener);
     void removeWindowEventListener(xcb_window_t id);
     QXcbWindowEventListener *windowEventListenerFromId(xcb_window_t id);
     QXcbWindow *platformWindowFromId(xcb_window_t id);
 
+    xcb_generic_event_t *checkEvent(int type);
     template<typename T>
     inline xcb_generic_event_t *checkEvent(T &checker);
 
     typedef bool (*PeekFunc)(QXcbConnection *, xcb_generic_event_t *);
     void addPeekFunc(PeekFunc f);
 
-    // Peek at all queued events
-    qint32 generatePeekerId();
-    bool removePeekerId(qint32 peekerId);
-    enum PeekOption { PeekDefault = 0, PeekFromCachedIndex = 1 }; // see qx11info_x11.h
-    Q_DECLARE_FLAGS(PeekOptions, PeekOption)
-    typedef bool (*PeekerCallback)(xcb_generic_event_t *event, void *peekerData);
-    bool peekEventQueue(PeekerCallback peeker, void *peekerData = nullptr,
-                        PeekOptions option = PeekDefault, qint32 peekerId = -1);
-
     inline xcb_timestamp_t time() const { return m_time; }
     inline void setTime(xcb_timestamp_t t) { if (t > m_time) m_time = t; }
 
     inline xcb_timestamp_t netWmUserTime() const { return m_netWmUserTime; }
     inline void setNetWmUserTime(xcb_timestamp_t t) { if (t > m_netWmUserTime) m_netWmUserTime = t; }
 
-    bool hasXFixes() const { return has_xfixes; }
+    bool hasGLX() const { return has_glx_extension; }
+    bool hasXFixes() const { return xfixes_first_event > 0; }
     bool hasXShape() const { return has_shape_extension; }
     bool hasXRandr() const { return has_randr_extension; }
     bool hasInputShape() const { return has_input_shape; }
+    bool hasTouchWithoutMouseEmulation() const { return has_touch_without_mouse_emulation; }
     bool hasXKB() const { return has_xkb; }
-    bool hasXRender(int major = -1, int minor = -1) const
-    {
-        if (has_render_extension && major != -1 && minor != -1)
-            return m_xrenderVersion >= qMakePair(major, minor);
 
-        return has_render_extension;
-    }
-    bool hasXInput2() const { return m_xi2Enabled; }
-    bool hasShm() const { return has_shm; }
-    bool hasShmFd() const { return has_shm_fd; }
-
-    bool threadedEventHandling() const { return m_reader->isRunning(); }
+    bool supportsThreadedRendering() const { return m_reader->isRunning(); }
 
     xcb_timestamp_t getTimestamp();
-    xcb_window_t getSelectionOwner(xcb_atom_t atom) const;
-    xcb_window_t getQtSelectionOwner();
 
-    void setButtonState(Qt::MouseButton button, bool down);
-    Qt::MouseButtons buttonState() const { return m_buttonState; }
-    Qt::MouseButton button() const { return m_button; }
-    Qt::MouseButton translateMouseButton(xcb_button_t s);
+    Qt::MouseButtons buttons() const { return m_buttons; }
 
     QXcbWindow *focusWindow() const { return m_focusWindow; }
-    void setFocusWindow(QWindow *);
-    QXcbWindow *mouseGrabber() const { return m_mouseGrabber; }
-    void setMouseGrabber(QXcbWindow *);
-    QXcbWindow *mousePressWindow() const { return m_mousePressWindow; }
-    void setMousePressWindow(QXcbWindow *);
+    void setFocusWindow(QXcbWindow *);
 
     QByteArray startupId() const { return m_startupId; }
     void setStartupId(const QByteArray &nextId) { m_startupId = nextId; }
@@ -516,35 +463,17 @@ public:
 
     QXcbNativeInterface *nativeInterface() const { return m_nativeInterface; }
 
-    QXcbSystemTrayTracker *systemTrayTracker() const;
-    static bool xEmbedSystemTrayAvailable();
-    static bool xEmbedSystemTrayVisualHasAlphaChannel();
+    QXcbSystemTrayTracker *systemTrayTracker();
 
-#if QT_CONFIG(xinput2)
-    void xi2SelectStateEvents();
-    void xi2SelectDeviceEvents(xcb_window_t window);
-    void xi2SelectDeviceEventsCompatibility(xcb_window_t window);
-    bool xi2SetMouseGrabEnabled(xcb_window_t w, bool grab);
-    bool xi2MouseEventsDisabled() const;
-    bool isAtLeastXI21() const { return m_xi2Enabled && m_xi2Minor >= 1; }
-    bool isAtLeastXI22() const { return m_xi2Enabled && m_xi2Minor >= 2; }
-    Qt::MouseButton xiToQtMouseButton(uint32_t b);
-#ifdef XCB_USE_XINPUT21
-    void xi2UpdateScrollingDevices();
-#endif
-#ifdef XCB_USE_XINPUT22
-    bool startSystemMoveResizeForTouchBegin(xcb_window_t window, const QPoint &point, int corner);
-    bool isTouchScreen(int id);
-#endif
+#ifdef XCB_USE_XINPUT2
+    void handleEnterEvent(const xcb_enter_notify_event_t *);
 #endif
+
     QXcbEventReader *eventReader() const { return m_reader; }
 
     bool canGrab() const { return m_canGrabServer; }
-
-    QXcbGlIntegration *glIntegration() const { return m_glIntegration; }
-
 protected:
-    bool event(QEvent *e) override;
+    bool event(QEvent *e) Q_DECL_OVERRIDE;
 
 public slots:
     void flush() { xcb_flush(m_connection); }
@@ -555,194 +484,146 @@ private slots:
 private:
     void initializeAllAtoms();
     void sendConnectionEvent(QXcbAtom::Atom atom, uint id = 0);
-    void initializeShm();
+    void initializeGLX();
     void initializeXFixes();
     void initializeXRender();
     void initializeXRandr();
-    void initializeXinerama();
     void initializeXShape();
     void initializeXKB();
     void handleClientMessageEvent(const xcb_client_message_event_t *event);
-    QXcbScreen* findScreenForCrtc(xcb_window_t rootWindow, xcb_randr_crtc_t crtc) const;
-    QXcbScreen* findScreenForOutput(xcb_window_t rootWindow, xcb_randr_output_t output) const;
-    QXcbVirtualDesktop* virtualDesktopForRootWindow(xcb_window_t rootWindow) const;
-    void updateScreens(const xcb_randr_notify_event_t *event);
-    bool checkOutputIsPrimary(xcb_window_t rootWindow, xcb_randr_output_t output);
-    void updateScreen(QXcbScreen *screen, const xcb_randr_output_change_t &outputChange);
-    QXcbScreen *createScreen(QXcbVirtualDesktop *virtualDesktop,
-                             const xcb_randr_output_change_t &outputChange,
-                             xcb_randr_get_output_info_reply_t *outputInfo);
-    void destroyScreen(QXcbScreen *screen);
-    void initializeScreens();
-    bool compressEvent(xcb_generic_event_t *event, int currentIndex, QXcbEventArray *eventqueue) const;
+    QXcbScreen* findOrCreateScreen(QList<QXcbScreen *>& newScreens, int screenNumber,
+        xcb_screen_t* xcbScreen, xcb_randr_get_output_info_reply_t *output = NULL);
+    void updateScreens();
+    void handleButtonPress(xcb_generic_event_t *event);
+    void handleButtonRelease(xcb_generic_event_t *event);
 
-    bool m_xi2Enabled = false;
-#if QT_CONFIG(xinput2)
-    int m_xi2Minor = -1;
+    bool m_xi2Enabled;
+    int m_xi2Minor;
+#ifdef XCB_USE_XINPUT2
     void initializeXInput2();
-    void xi2SetupDevice(void *info, bool removeExisting = true);
+    void finalizeXInput2();
     void xi2SetupDevices();
-    struct TouchDeviceData {
-        QTouchDevice *qtTouchDevice = nullptr;
-        QHash<int, QWindowSystemInterface::TouchPoint> touchPoints;
-        QHash<int, QPointF> pointPressedPosition; // in screen coordinates where each point was pressed
-        struct ValuatorClassInfo {
-            double min = 0;
-            double max = 0;
-            int number = -1;
-            QXcbAtom::Atom label;
-        };
-        QVector<ValuatorClassInfo> valuatorInfo;
-
-        // Stuff that is relevant only for touchpads
-        QPointF firstPressedPosition;        // in screen coordinates where the first point was pressed
-        QPointF firstPressedNormalPosition;  // device coordinates (0 to 1, 0 to 1) where the first point was pressed
-        QSizeF size;                         // device size in mm
-        bool providesTouchOrientation = false;
-    };
-    TouchDeviceData *populateTouchDevices(void *info);
-    TouchDeviceData *touchDeviceForId(int id);
+    XInput2TouchDeviceData *touchDeviceForId(int id);
     void xi2HandleEvent(xcb_ge_event_t *event);
-    void xi2HandleHierarchyEvent(void *event);
-    void xi2HandleDeviceChangedEvent(void *event);
+    void xi2HandleHierachyEvent(void *event);
     int m_xiOpCode, m_xiEventBase, m_xiErrorBase;
-#ifdef XCB_USE_XINPUT22
-    void xi2ProcessTouch(void *xiDevEvent, QXcbWindow *platformWindow);
-#endif // XCB_USE_XINPUT22
-#if QT_CONFIG(tabletevent)
+#ifndef QT_NO_TABLETEVENT
     struct TabletData {
-        int deviceId = 0;
-        QTabletEvent::PointerType pointerType = QTabletEvent::UnknownPointer;
-        QTabletEvent::TabletDevice tool = QTabletEvent::Stylus;
-        Qt::MouseButtons buttons = 0;
-        qint64 serialId = 0;
-        bool inProximity = false;
+        TabletData() : deviceId(0), pointerType(QTabletEvent::UnknownPointer),
+            tool(QTabletEvent::Stylus), buttons(0), serialId(0), inProximity(false) { }
+        int deviceId;
+        QTabletEvent::PointerType pointerType;
+        QTabletEvent::TabletDevice tool;
+        Qt::MouseButtons buttons;
+        qint64 serialId;
+        bool inProximity;
         struct ValuatorClassInfo {
-            double minVal = 0;
-            double maxVal = 0;
-            double curVal = 0;
-            int number = -1;
+            ValuatorClassInfo() : minVal(0.), maxVal(0.), curVal(0.) { }
+            double minVal;
+            double maxVal;
+            double curVal;
+            int number;
         };
         QHash<int, ValuatorClassInfo> valuatorInfo;
     };
-    friend class QTypeInfo<TabletData>;
-    friend class QTypeInfo<TabletData::ValuatorClassInfo>;
-    bool xi2HandleTabletEvent(const void *event, TabletData *tabletData);
-    void xi2ReportTabletEvent(const void *event, TabletData *tabletData);
+    bool xi2HandleTabletEvent(void *event, TabletData *tabletData);
+    void xi2ReportTabletEvent(TabletData &tabletData, void *event);
     QVector<TabletData> m_tabletData;
-    TabletData *tabletDataForDevice(int id);
-#endif // QT_CONFIG(tabletevent)
+#endif
     struct ScrollingDevice {
-        int deviceId = 0;
-        int verticalIndex = 0;
-        int horizontalIndex = 0;
-        double verticalIncrement = 0;
-        double horizontalIncrement = 0;
-        Qt::Orientations orientations = 0;
-        Qt::Orientations legacyOrientations = 0;
+        ScrollingDevice() : deviceId(0), verticalIndex(0), horizontalIndex(0), orientations(0), legacyOrientations(0) { }
+        int deviceId;
+        int verticalIndex, horizontalIndex;
+        double verticalIncrement, horizontalIncrement;
+        Qt::Orientations orientations;
+        Qt::Orientations legacyOrientations;
         QPointF lastScrollPosition;
     };
-    QHash<int, ScrollingDevice> m_scrollingDevices;
-#ifdef XCB_USE_XINPUT21
     void xi2HandleScrollEvent(void *event, ScrollingDevice &scrollingDevice);
-    void xi2UpdateScrollingDevice(ScrollingDevice &scrollingDevice);
-    ScrollingDevice *scrollingDeviceForId(int id);
+    QHash<int, ScrollingDevice> m_scrollingDevices;
+#endif // XCB_USE_XINPUT2
+
+#if defined(XCB_USE_XINPUT2)
+    static bool xi2GetValuatorValueIfSet(void *event, int valuatorNum, double *value);
+    static bool xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event, int opCode);
 #endif
 
-    static bool xi2GetValuatorValueIfSet(const void *event, int valuatorNum, double *value);
-    static void xi2PrepareXIGenericDeviceEvent(xcb_ge_event_t *event);
-#endif
+    xcb_connection_t *m_connection;
+    const xcb_setup_t *m_setup;
+    bool m_canGrabServer;
 
-    xcb_connection_t *m_connection = nullptr;
-    const xcb_setup_t *m_setup = nullptr;
-    const bool m_canGrabServer;
-    const xcb_visualid_t m_defaultVisualId;
-
-    QList<QXcbVirtualDesktop *> m_virtualDesktops;
     QList<QXcbScreen *> m_screens;
-    int m_primaryScreenNumber = 0;
+    int m_primaryScreenNumber;
 
     xcb_atom_t m_allAtoms[QXcbAtom::NAtoms];
 
-    xcb_timestamp_t m_time = XCB_CURRENT_TIME;
-    xcb_timestamp_t m_netWmUserTime = XCB_CURRENT_TIME;
+    xcb_timestamp_t m_time;
+    xcb_timestamp_t m_netWmUserTime;
 
     QByteArray m_displayName;
 
-    QXcbKeyboard *m_keyboard = nullptr;
+    QXcbKeyboard *m_keyboard;
 #ifndef QT_NO_CLIPBOARD
-    QXcbClipboard *m_clipboard = nullptr;
+    QXcbClipboard *m_clipboard;
 #endif
 #ifndef QT_NO_DRAGANDDROP
-    QXcbDrag *m_drag = nullptr;
+    QXcbDrag *m_drag;
 #endif
     QScopedPointer<QXcbWMSupport> m_wmSupport;
-    QXcbNativeInterface *m_nativeInterface = nullptr;
+    QXcbNativeInterface *m_nativeInterface;
 
-#if QT_CONFIG(xcb_xlib)
-    void *m_xlib_display = nullptr;
+#if defined(XCB_USE_XLIB)
+    void *m_xlib_display;
+#endif
+    QXcbEventReader *m_reader;
+#if defined(XCB_USE_XINPUT2)
+    QHash<int, QWindowSystemInterface::TouchPoint> m_touchPoints;
+    QHash<int, XInput2TouchDeviceData*> m_touchDevices;
+#endif
+#if defined(XCB_USE_EGL)
+    void *m_egl_display;
+    bool m_has_egl;
+#endif
+#ifdef Q_XCB_DEBUG
+    struct CallInfo {
+        int sequence;
+        QByteArray file;
+        int line;
+    };
+    QVector<CallInfo> m_callLog;
+    QMutex m_callLogMutex;
+    void log(const char *file, int line, int sequence);
+    template <typename cookie_t>
+    friend cookie_t q_xcb_call_template(const cookie_t &cookie, QXcbConnection *connection, const char *file, int line);
 #endif
-    QXcbEventReader *m_reader = nullptr;
 
-#if QT_CONFIG(xinput2)
-    QHash<int, TouchDeviceData> m_touchDevices;
-#ifdef XCB_USE_XINPUT22
-    struct StartSystemMoveResizeInfo {
-        xcb_window_t window = XCB_NONE;
-        uint16_t deviceid;
-        uint32_t pointid;
-        int corner;
-    } m_startSystemMoveResizeInfo;
-#endif
-#endif
     WindowMapper m_mapper;
 
     QVector<PeekFunc> m_peekFuncs;
 
-    uint32_t xfixes_first_event = 0;
-    uint32_t xrandr_first_event = 0;
-    uint32_t xkb_first_event = 0;
+    uint32_t xfixes_first_event;
+    uint32_t xrandr_first_event;
+    uint32_t xkb_first_event;
+    uint32_t glx_first_event;
 
-    bool has_xfixes = false;
-    bool has_xinerama_extension = false;
-    bool has_shape_extension = false;
-    bool has_randr_extension = false;
+    bool has_glx_extension;
+    bool has_shape_extension;
+    bool has_randr_extension;
     bool has_input_shape;
-    bool has_xkb = false;
-    bool has_render_extension = false;
-    bool has_shm = false;
-    bool has_shm_fd = false;
+    bool has_touch_without_mouse_emulation;
+    bool has_xkb;
 
-    QPair<int, int> m_xrenderVersion;
+    Qt::MouseButtons m_buttons;
 
-    Qt::MouseButtons m_buttonState = 0;
-    Qt::MouseButton m_button = Qt::NoButton;
+    QXcbWindow *m_focusWindow;
 
-    QXcbWindow *m_focusWindow = nullptr;
-    QXcbWindow *m_mouseGrabber = nullptr;
-    QXcbWindow *m_mousePressWindow = nullptr;
-
-    xcb_window_t m_clientLeader = 0;
     QByteArray m_startupId;
-    QXcbSystemTrayTracker *m_systemTrayTracker = nullptr;
-    QXcbGlIntegration *m_glIntegration = nullptr;
-    bool m_xiGrab = false;
-    QVector<int> m_xiMasterPointerIds;
+    QXcbSystemTrayTracker *m_systemTrayTracker;
 
-    xcb_window_t m_qtSelectionOwner = 0;
-
-    bool m_mainEventLoopFlushedQueue = false;
-    qint32 m_peekerIdSource = 0;
-    bool m_peekerIndexCacheDirty = false;
-    QHash<qint32, qint32> m_peekerToCachedIndex;
     friend class QXcbEventReader;
 };
-#if QT_CONFIG(xinput2)
-#if QT_CONFIG(tabletevent)
-Q_DECLARE_TYPEINFO(QXcbConnection::TabletData::ValuatorClassInfo, Q_PRIMITIVE_TYPE);
-Q_DECLARE_TYPEINFO(QXcbConnection::TabletData, Q_MOVABLE_TYPE);
-#endif
-#endif
+
+#define DISPLAY_FROM_XCB(object) ((Display *)(object->connection()->xlib_display()))
 
 template<typename T>
 xcb_generic_event_t *QXcbConnection::checkEvent(T &checker)
@@ -771,34 +652,26 @@ private:
     QXcbConnection *m_connection;
 };
 
-#define Q_XCB_REPLY_CONNECTION_ARG(connection, ...) connection
+#ifdef Q_XCB_DEBUG
+template <typename cookie_t>
+cookie_t q_xcb_call_template(const cookie_t &cookie, QXcbConnection *connection, const char *file, int line)
+{
+    connection->log(file, line, cookie.sequence);
+    return cookie;
+}
+#define Q_XCB_CALL(x) q_xcb_call_template(x, connection(), __FILE__, __LINE__)
+#define Q_XCB_CALL2(x, connection) q_xcb_call_template(x, connection, __FILE__, __LINE__)
+#define Q_XCB_NOOP(c) q_xcb_call_template(xcb_no_operation(c->xcb_connection()), c, __FILE__, __LINE__);
+#else
+#define Q_XCB_CALL(x) x
+#define Q_XCB_CALL2(x, connection) x
+#define Q_XCB_NOOP(c)
+#endif
 
-struct QStdFreeDeleter {
-    void operator()(void *p) const Q_DECL_NOTHROW { return std::free(p); }
-};
 
-#define Q_XCB_REPLY(call, ...) \
-    std::unique_ptr<call##_reply_t, QStdFreeDeleter>( \
-        call##_reply(Q_XCB_REPLY_CONNECTION_ARG(__VA_ARGS__), call(__VA_ARGS__), nullptr) \
-    )
-
-#define Q_XCB_REPLY_UNCHECKED(call, ...) \
-    std::unique_ptr<call##_reply_t, QStdFreeDeleter>( \
-        call##_reply(Q_XCB_REPLY_CONNECTION_ARG(__VA_ARGS__), call##_unchecked(__VA_ARGS__), nullptr) \
-    )
-
-template <typename T>
-union q_padded_xcb_event {
-  T event;
-  char padding[32];
-};
-
-// The xcb_send_event() requires all events to have 32 bytes. It calls memcpy() on the
-// passed in event. If the passed in event is less than 32 bytes, memcpy() reaches into
-// unrelated memory.
-#define Q_DECLARE_XCB_EVENT(event_var, event_type) \
-    q_padded_xcb_event<event_type> store = {}; \
-    auto &event_var = store.event;
+#if defined(XCB_USE_EGL)
+#define EGL_DISPLAY_FROM_XCB(object) ((EGLDisplay)(object->connection()->egl_display()))
+#endif
 
 QT_END_NAMESPACE
 
